@@ -54,7 +54,6 @@ class DocumentMeta(type):
         
         if new_class.config_extra_fields not in ['error', 'ignore']:
             raise DocumentException("config_extra_fields must be one of: 'error', 'ignore'")
-
         
         # 1. Set up links between fields and the document class
         for name, value in class_dict.iteritems():
@@ -70,11 +69,6 @@ class DocumentMeta(type):
             if not isinstance(field, QueryField):
                 continue
             new_class._fields[name] = field.get_type()
-            # field.set_owner(new_class)
-        
-        # Create a query field instance for use in query expressions
-        # new_class.f = QueryFieldSet(new_class, new_class.get_fields())
-        
         return new_class
 
 class Document(object):
@@ -130,6 +124,11 @@ class Document(object):
                     raise ExtraValueException(k)
     
     def get_dirty_ops(self):
+        ''' Returns a dict with the update operations necessary to make the 
+            changes to this object to the database version.  It is mainly used
+            internally for :func:`~mongoalchemy.session.Session.update` but
+            may be useful for diagnostic purposes as well.
+        '''
         update_expression = {}
         for name, field in self.get_fields().iteritems():
             dirty_ops = field.dirty_ops(self)
@@ -164,11 +163,11 @@ class Document(object):
     
     @classmethod
     def get_collection_name(cls):
-        ''' Returns the collection name used by the class.  If the ``_collection_name``
+        ''' Returns the collection name used by the class.  If the ``config_collection_name``
             attribute is set it is used, otherwise the name of the class is used.'''
-        if not hasattr(cls, '_collection_name'):
+        if not hasattr(cls, 'config_collection_name'):
             return cls.__name__
-        return cls._collection_name
+        return cls.config_collection_name
     
     @classmethod
     def get_indexes(cls):
@@ -216,7 +215,7 @@ class Document(object):
         self.mongo_id = id
     
     def wrap(self):
-        '''Returns a transformation of this document into a form suitable to 
+        ''' Returns a transformation of this document into a form suitable to 
             be saved into a mongo database.  This is done by using the ``wrap()``
             methods of the underlying fields to set values.'''
         res = {}
@@ -237,6 +236,10 @@ class Document(object):
     
     @classmethod
     def validate_unwrap(cls, obj, fields=None):
+        ''' Attempts to unwrap the document, and raises a BadValueException if
+            the operation fails. A TODO is to make this function do the checks
+            without actually doing the (potentially expensive) 
+            deserialization'''
         try:
             cls.unwrap(obj, fields=fields)
         except Exception, e:
@@ -330,6 +333,8 @@ class DocumentField(Field):
         self.document_class = document_class
     
     def dirty_ops(self, instance):
+        ''' Returns a dict of the operations needed to update this object.  
+            See :func:`Document.get_dirty_ops` for more details.'''
         try:
             document = getattr(instance, self.name)
         except AttributeError:
@@ -348,6 +353,8 @@ class DocumentField(Field):
         return ret
     
     def subfields(self):
+        ''' Returns the fields that can be retrieved from the enclosed 
+            document.  This function is mainly used internally'''
         return self.document_class.get_fields()
     
     def sub_type(self):
