@@ -64,7 +64,7 @@ from copy import deepcopy
 
 from mongoalchemy.util import UNSET
 from mongoalchemy.query_expression import QueryField
-from mongoalchemy.exceptions import BadValueException, FieldNotRetrieved, InvalidConfigException
+from mongoalchemy.exceptions import BadValueException, FieldNotRetrieved, InvalidConfigException, BadFieldSpecification
 
 SCALAR_MODIFIERS = set(['$set', '$unset'])
 NUMBER_MODIFIERS = SCALAR_MODIFIERS | set(['$inc'])
@@ -74,32 +74,53 @@ ANY_MODIFIER = LIST_MODIFIERS | NUMBER_MODIFIERS
 class FieldMeta(type):
     def __new__(mcs, classname, bases, class_dict):
         
-        def validation_wrapper(fun):
+        def validation_wrapper(fun, kind):
             def wrapped(self, value, *args, **kwds):
+                # Handle None
                 if self._allow_none and value == None:
                     return
-                return fun(self, value, *args, **kwds)
+                # Standard Field validation
+                fun(self, value, *args, **kwds)
+                
+                # Universal user-supplied validator
+                if self.validator:
+                    if self.validator(value) == False:
+                        self._fail_validation(value, 'user-supplied validator failed')
+                
+                if kind == 'unwrap' and self.unwrap_validator:
+                    if self.unwrap_validator(value) == False:
+                        self._fail_validation(value, 'user-supplied unwrap_validator failed')
+
+                elif kind == 'wrap' and self.wrap_validator:
+                    if self.wrap_validator(value) == False:
+                        self._fail_validation(value, 'user-supplied wrap_validator failed')
+            
             functools.update_wrapper(wrapped, fun, ('__name__', '__doc__'))
             return wrapped
         
         if 'validate_wrap' in class_dict:
-            class_dict['validate_wrap'] = validation_wrapper(class_dict['validate_wrap'])
+            class_dict['validate_wrap'] = validation_wrapper(class_dict['validate_wrap'], 'wrap')
         
         if 'validate_unwrap' in class_dict:
-            class_dict['validate_unwrap'] = validation_wrapper(class_dict['validate_unwrap'])
+            class_dict['validate_unwrap'] = validation_wrapper(class_dict['validate_unwrap'], 'unwrap')
         
         # Create Class
         return type.__new__(mcs, classname, bases, class_dict)
 
 class Field(object):
     auto = False
+    
+    #: If this kind of field can have sub-fields, this attribute should be True
     has_subfields = False
     
+    no_real_attributes = False  # used for free-form queries.  
+    
     __metaclass__ = FieldMeta
     
     valid_modifiers = SCALAR_MODIFIERS
     
-    def __init__(self, required=True, default=UNSET, db_field=None, allow_none=False, on_update='$set'):
+    def __init__(self, required=True, default=UNSET, db_field=None, allow_none=False, on_update='$set', 
+            validator=None, unwrap_validator=None, wrap_validator=None):
         '''
         **Parameters**:
             * required: The field must be passed when constructing a document (optional. default: ``True``)
@@ -107,11 +128,24 @@ class Field(object):
             * db_field: name to use when saving or loading this field from the database \
                 (optional.  default is the name the field is assigned to on a documet)
             * allow_none: allow ``None`` as a value (optional. default: False)
+            * validator: a callable which will be called on objects when wrapping/unwrapping
+            * unwrap_validator: a callable which will be called on objects when unwrapping
+            * wrap_validator: a callable which will be called on objects when wrapping
+        
+        The general validator is called after the field's validator, but before 
+        either of the wrap/unwrap versions.  The validator should raise a BadValueException
+        if it fails, but if it returns False the field will raise an exception with
+        a generic message.
+        
         '''
         self.__db_field = db_field
         self.__value = UNSET
         self.__update_op = UNSET
         
+        self.validator = validator
+        self.unwrap_validator = unwrap_validator
+        self.wrap_validator = wrap_validator
+        
         self._allow_none = allow_none
         self._owner = None
         
@@ -168,6 +202,11 @@ class Field(object):
             return self.__db_field
         return self.name
     
+    def wrap_value(self, value):
+        ''' Wrap ``value`` for use as the value in a Mongo query, for example 
+            in $in'''
+        return self.wrap(value)
+    
     def _set_name(self, name):
         self.name = name
     
@@ -225,7 +264,7 @@ class Field(object):
         raise BadValueException(self.name, value, 'Value is not an instance of %s (got: %s)' % (types, got))
     
     def is_valid_wrap(self, value):
-        '''Returns whether ``value`` is a valid value to wrap.
+        ''' Returns whether ``value`` is a valid value to wrap.
             Raises ``NotImplementedError`` in the base class.
         
             **Parameters**: 
@@ -251,7 +290,7 @@ class Field(object):
         return True
 
 class PrimitiveField(Field):
-    '''Primitive fields are fields where a single constructor can be used
+    ''' Primitive fields are fields where a single constructor can be used
         for wrapping and unwrapping an object.'''
     
     valid_modifiers = SCALAR_MODIFIERS
@@ -425,6 +464,7 @@ class TupleField(Field):
     
     def wrap(self, value):
         ''' Validate and then wrap ``value`` for insertion.
+            
             **Parameters**
                 * value: the tuple (or list) to wrap
         '''
@@ -436,6 +476,7 @@ class TupleField(Field):
     
     def unwrap(self, value):
         ''' Validate and then unwrap ``value`` for object creation.
+            
             **Parameters**
                 * value: list returned from the database.  
         '''
@@ -477,8 +518,7 @@ class EnumField(Field):
             self._fail_validation(value, 'Value was not in the enum values')
     
     def validate_unwrap(self, value):
-        ''' 
-            Checks that value is valid for `EnumField.item_type`.  
+        ''' Checks that value is valid for `EnumField.item_type`.  
             
             .. note ::
                 Since checking the value itself is not possible until is is 
@@ -486,7 +526,8 @@ class EnumField(Field):
         self.item_type.validate_unwrap(value)
     
     def wrap(self, value):
-        '''Validate and wrap value using the wrapping function from ``EnumField.item_type``
+        ''' Validate and wrap value using the wrapping function from 
+            ``EnumField.item_type``
         '''
         self.validate_wrap(value)
         return self.item_type.wrap(value)
@@ -524,6 +565,29 @@ class SequenceField(Field):
         if not isinstance(item_type, Field):
             raise BadFieldSpecification("List item_type is not a field!")
     
+    @property
+    def has_subfields(self):
+        ''' Returns True if the sequence's value type has subfields. '''
+        return self.item_type.has_subfields
+    
+    def subfields(self):
+        ''' Returns the names of the value type's sub-fields'''
+        return self.item_type.subfields()
+    
+    def wrap_value(self, value):
+        ''' A function used to wrap a value used in a comparison.  It will 
+            first try to wrap as the sequence's sub-type, and then as the 
+            sequence itself'''
+        try:
+            return self.item_type.wrap_value(value)
+        except BadValueException:
+            pass
+        try:
+            return self.wrap(value)
+        except BadValueException:
+            pass
+        self._fail_validation(value, 'Could not wrap value as the correct type.  Tried %s and %s' % (self.item_type, self))
+    
     def child_type(self):
         ''' Returns the :class:`Field` instance used for items in the sequence'''
         return self.item_type
@@ -557,7 +621,7 @@ class SequenceField(Field):
             self._validate_child_unwrap(v)
 
 class ListField(SequenceField):
-    '''Field representing a python list.
+    ''' Field representing a python list.
         
         .. seealso:: :class:`SequenceField`'''
     def _validate_wrap_type(self, value):
@@ -577,7 +641,7 @@ class ListField(SequenceField):
         return [self.item_type.unwrap(v) for v in value]
 
 class SetField(SequenceField):
-    '''Field representing a python set.
+    ''' Field representing a python set.
         
         .. seealso:: :class:`SequenceField`'''
     def _validate_wrap_type(self, value):
@@ -623,7 +687,7 @@ class AnythingField(Field):
         pass
 
 class ObjectIdField(Field):
-    '''pymongo Object ID object.  Currently this is probably too strict.  A 
+    ''' pymongo Object ID object.  Currently this is probably too strict.  A 
         string version of an ObjectId should also be acceptable'''
     
     valid_modifiers = SCALAR_MODIFIERS 
@@ -731,9 +795,11 @@ class DictField(Field):
 
 class KVField(DictField):
     ''' Like a DictField, except it allows arbitrary keys.  The DB Format for 
-        a ``KVField`` is ``[ { 'k' : key, 'v' : value }, ...]``.  This will eventually
-        makes it possible to have an index on the keys and values.
-    '''
+        a ``KVField`` is ``[ { 'k' : key, 'v' : value }, ...]``.  Queries on 
+        keys and values. can be done with ``.k`` and ``.v`` '''
+    #: If this kind of field can have sub-fields, this attribute should be True
+    has_subfields = True
+    
     def __init__(self, key_type, value_type, **kwargs):
         '''
             **Parameters**:
@@ -741,12 +807,25 @@ class KVField(DictField):
                 * value_type: the Field type to use for the values
         '''
         super(DictField, self).__init__(**kwargs)
-        self.key_type = key_type
-        self.value_type = value_type
+        
         if not isinstance(key_type, Field):
             raise BadFieldSpecification("KVField key type is not a field!")
         if not isinstance(value_type, Field):
             raise BadFieldSpecification("KVField value type is not a field!")
+        self.key_type = key_type
+        self.key_type.name = 'k'
+        
+        self.value_type = value_type
+        self.value_type.name = 'v'
+        
+    def subfields(self):
+        '''Returns the k and v subfields, which can be accessed to do queries
+            based on either of them
+        '''
+        return {
+            'k' : self.key_type,
+            'v' : self.value_type,
+        }
     
     def _validate_key_wrap(self, key):
         try:
@@ -779,22 +858,6 @@ class KVField(DictField):
                 self.value_type.validate_unwrap(v)
             except BadValueException, bve:
                 self._fail_validation(value, 'Bad value for KFVield value %s' % k, cause=bve)
-            
-            # try:
-            #     self.key_type.validate_unwrap(v)
-            # except BadValueException, bve:
-            #     self._fail_validation(value, 'Bad value for key %s' % k, cause=bve)
-            # 
-            # try:
-            #     self.value_type.validate_unwrap(v)
-            # except BadValueException, bve:
-            #     self._fail_validation(value, 'Bad value for KVField valye key %s' % k, cause=bve)
-            # 
-            # 
-            # if not self.key_type.is_valid_unwrap(k):
-            #     return False
-            # if not self.value_type.is_valid_unwrap(v):
-            #     return False
         return True
     
     def wrap(self, value):
@@ -826,23 +889,33 @@ class KVField(DictField):
         return ret
 
 class ComputedField(Field):
-    ''' A computed field is generated based on an object's other values.  
+    ''' A computed field is generated based on an object's other values.  It
+        will generally be created with the @computed_field decorator, but
+        can be passed an arbitrary function.
+        
+        The function should take a dict which will contains keys with the names
+        of the dependencies mapped to their values.
+        
+        The computed value is recalculated every the field is accessed unless 
+        the one_time field is set to True.
+        
+        Example::
+        
+            >>> class SomeDoc(Document):
+            ...     @computed_field
+            ...     def last_modified(obj):
+            ...         return datetime.datetime.utcnow()
         
-        the unwrap function takes a dictionary of K/V pairs of the 
-        dependencies.  Since dependencies are declared in the class 
-        definition all of the dependencies for a computed field should be
-        in the class definition before the computed field itself.
         
         .. warning::
-            The computed field interacts weirdly with documents right now, 
-            especially with respect to partial loading.  If using this class
-            watch out for strange behaviour
+            The computed field interacts in an undefined way with partially loaded 
+            documents right now.  If using this class watch out for strange behaviour.
     '''
     
     valid_modifiers = SCALAR_MODIFIERS
     
     auto = True
-    def __init__(self, computed_type, fun, deps=None, **kwargs):
+    def __init__(self, computed_type, fun, one_time=False, deps=None, **kwargs):
         '''
             **Parameters**:
                 * fun: the function to compute the value of the computed field
@@ -856,13 +929,26 @@ class ComputedField(Field):
             deps = set()
         self.deps = set(deps)
         self.fun = fun
+        self.one_time = one_time
+        self.__cached_value = UNSET
     
     def __get__(self, instance, owner):
+        # class method
         if type(instance) == type(None):
             return QueryField(self)
-        # TODO: dirty cache indictor + check a field option for never caching
-        return self.compute_value(instance)
+        
+        if self.name in instance._field_values and self.one_time:
+            return instance._field_values[self.name]
+        computed_value = self.compute_value(instance)
+        if self.one_time:
+            instance._field_values[self.name] = computed_value
+        return computed_value
     
+    def __set__(self, instance, value):
+        if self.name in instance._field_values and self.one_time:
+            raise BadValueException(self.name, value, 'Cannot set a one-time field once it has been set')
+        super(ComputedField, self).__set__(instance, value)
+
     def dirty_ops(self, instance):
         dirty = False
         for dep in self.deps:
@@ -888,7 +974,12 @@ class ComputedField(Field):
         except BadValueException, bve:
             self._fail_validation(value, 'Computed Function return a bad value', cause=bve)
         return value
-
+    
+    def wrap_value(self, value):
+        ''' A function used to wrap a value used in a comparison.  It will 
+            first try to wrap as the sequence's sub-type, and then as the 
+            sequence itself'''
+        return self.computed_type.wrap_value(value)
     
     def validate_wrap(self, value):
         '''Check that ``value`` is valid for unwrapping with ``ComputedField.computed_type``'''
@@ -913,14 +1004,6 @@ class ComputedField(Field):
         ''' Validates ``value`` and unwraps it with ``ComputedField.computed_type``'''
         self.validate_unwrap(value)
         return self.computed_type.unwrap(value)
-    
-    def __set__(self, instance, value):
-        if self.is_valid_wrap(value):
-            self.__computed_value = value
-            return
-        # TODO: this line should be impossible to reach, but I'd like an 
-        # exception just in case, but then I can't have full coverage!
-        # raise BadValueException('Tried to set a computed field to an illegal value: %s' % value)
 
 class computed_field(object):
     def __init__(self, computed_type, deps=None, **kwargs):
@@ -930,9 +1013,3 @@ class computed_field(object):
     
     def __call__(self, fun):
         return ComputedField(self.computed_type, fun, deps=self.deps, **self.kwargs)
-    
-class BadFieldSpecification(Exception):
-    '''An exception that is raised when there is an error in creating a 
-        field'''
-    pass
-
