@@ -168,6 +168,12 @@ def test_clone():
         pass
     assert count == 1
 
+def test_raw_query():
+    s = get_session()
+    s.clear_collection(T)
+    s.insert(T(i=3))
+    assert s.query(T).filter({'i':3}).one().i == 3
+
 @raises(FieldNotRetrieved)
 def test_field_filter_non_retrieved_field():
     s = get_session()
@@ -204,7 +210,7 @@ def repeated_field_query_test2():
 def nested_field_query_test():
     s = get_session()
     s.clear_collection(T, T2)
-    s.query(T2).filter(T2.t.i==3, T2.t.j==2)
+    assert s.query(T2).filter(T2.t.i==3, T2.t.j==2).query == {'t.i':3, 't.j':2}
 
 @known_failure
 @raises(Exception)
@@ -292,4 +298,13 @@ def qr_test_clone():
         it2.next()
     except StopIteration:
         pass
-    
\ No newline at end of file
+
+def test_resolve_fields():
+    class Resolver(Document):
+        i = IntField(db_field='j')
+        k = IntField()
+    s = get_session()
+    s.clear_collection(Resolver)
+    
+    q = s.query(Resolver).filter(Resolver.i.in_(6))
+    q = s.query(Resolver).set(Resolver.i, 6)
